__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import json
import csv
import os
import pyproj
from functools import partial
from planet import api
from planet.api import filters
from planet.api.auth import find_api_key
from shapely.geometry import shape
from shapely.ops import transform


try:
    PL_API_KEY = find_api_key()
except Exception as e:
    print("Failed to get Planet Key")
    sys.exit()

client = api.ClientV1(PL_API_KEY)


temp = {"coordinates": [], "type": "MultiPolygon"}
aoi = {"type": "Polygon", "coordinates": []}

# Area lists
ar = []
far = []
ovall = []

## Handle MultiPolygon
def multipoly(poly):
    multipoly_empty = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"id": 0},
                "geometry": {"type": "MultiPolygon", "coordinates": []},
            }
        ],
    }
    poly_list = []
    with open(poly) as jsonfile:
        data = json.load(jsonfile)
        if len(data["features"]) > 1:
            for things in data["features"]:
                poly_list.append(things["geometry"]["coordinates"])
            multipoly_empty["features"][0]["geometry"]["coordinates"] = poly_list
            return json.dumps(multipoly_empty)
        elif len(data["features"]) == 1:
            return data


# get coordinates list depth
def list_depth(dic, level=1):
    counter = 0
    str_dic = str(dic)
    if "[[[[[" in str_dic:
        counter += 2
    elif "[[[[" in str_dic:
        counter += 1
    elif "[[[" in str_dic:
        counter += 0
    return counter


# Function to use the client and then search
def idc(idlist, item, asset, geometry):
    if geometry is not None:
        try:
            if geometry.endswith(".geojson"):
                aoi_resp = multipoly(geometry)
                try:
                    for things in aoi_resp["features"]:
                        ovall.append(things["geometry"]["coordinates"])
                except Exception:
                    for things in json.loads(aoi_resp)["features"]:
                        ovall.append(things["geometry"]["coordinates"])
                # print(list_depth(ovall))
                aoi_geom = ovall
                if (
                    list_depth(ovall) == 2
                    and json.loads(aoi_resp)["features"][0]["geometry"]["type"]
                    == "MultiPolygon"
                ):
                    temp["coordinates"] = aoi_geom[0]
                elif (
                    list_depth(ovall) == 2
                    and json.loads(aoi_resp)["features"][0]["geometry"]["type"]
                    != "MultiPolygon"
                ):
                    aoi_geom = ovall[0][0]
                    temp["type"] = "Polygon"
                    temp["coordinates"] = aoi_geom
                elif list_depth(ovall) == 1:
                    aoi_geom = ovall[0]
                    temp["type"] = "Polygon"
                    temp["coordinates"] = aoi_geom
                elif list_depth(ovall) == 0:
                    aoi_geom = ovall
                    temp["type"] = "Polygon"
                    temp["coordinates"] = aoi_geom
                else:
                    print("Please check GeoJSON: Could not parse coordinates")
        except Exception as e:
            print("Could not parse geometry")
            print(e)
        l = []
        stbase = {"config": [], "field_name": "id", "type": "StringInFilter"}
        with open(idlist) as f:
            try:
                reader = csv.reader(f)
                for row in reader:
                    item_id = row[0]
                    if str(item_id.isalpha()) == "False":
                        l.append(item_id)
            except Exception as e:
                print(e)
        stbase["config"] = l
        # temp["coordinates"] = aoi_geom
        # sgeom = filters.geom_filter(temp)
        aoi_shape = shape(temp)
        asset_filter = filters.permission_filter("assets." + str(asset) + ":download")
        and_filter = filters.and_filter(asset_filter, stbase)
        item_types = [item]
        req = filters.build_search_request(and_filter, item_types)
        res = client.quick_search(req)
        num_lines = sum(1 for line in open(os.path.join(idlist.split(".")[0] + ".csv")))
        n = 1
        print("Checking assets in idlist...")
        for things in res.items_iter(
            num_lines
        ):  # A large number as max number to check against
            itemid = things["id"]
            # print('Processing '+str(n)+' of '+str(num_lines))
            n = n + 1
            footprint = things["geometry"]
            s = shape(footprint)
            if item.startswith("SkySat"):
                epsgcode = "3857"
            else:
                epsgcode = things["properties"]["epsg_code"]
            if aoi_shape.area > s.area:
                intersect = (s).intersection(aoi_shape)
            elif s.area >= aoi_shape.area:
                intersect = (aoi_shape).intersection(s)
            proj = pyproj.Transformer.from_proj(
                pyproj.Proj(4326), pyproj.Proj(epsgcode), always_xy=True
            ).transform
            if transform(proj, aoi_shape).area > transform(proj, s).area:
                ar.append(transform(proj, intersect).area / 1000000)
                far.append(transform(proj, s).area / 1000000)
            elif transform(proj, s).area > transform(proj, aoi_shape).area:
                ar.append(transform(proj, intersect).area / 1000000)
                far.append(transform(proj, s).area / 1000000)
    elif geometry is None:
        l = []
        stbase = {"config": [], "field_name": "id", "type": "StringInFilter"}
        with open(idlist) as f:
            try:
                reader = csv.reader(f)
                for row in reader:
                    item_id = row[0]
                    if str(item_id.isalpha()) == "False":
                        l.append(item_id)
            except Exception as e:
                print(e)
        stbase["config"] = l
        asset_filter = filters.permission_filter("assets." + str(asset) + ":download")
        and_filter = filters.and_filter(asset_filter, stbase)
        item_types = [item]
        req = filters.build_search_request(and_filter, item_types)
        res = client.quick_search(req)
        num_lines = sum(1 for line in open(os.path.join(idlist.split(".")[0] + ".csv")))
        n = 1
        print("Checking assets in idlist...")
        for things in res.items_iter(
            num_lines
        ):  # A large number as max number to check against
            itemid = things["id"]
            # print('Processing '+str(n)+' of '+str(num_lines))
            n = n + 1
            footprint = things["geometry"]
            s = shape(footprint)
            if item.startswith("SkySat"):
                epsgcode = "3857"
            else:
                epsgcode = things["properties"]["epsg_code"]
            proj = pyproj.Transformer.from_proj(
                pyproj.Proj(4326), pyproj.Proj(epsgcode), always_xy=True
            ).transform
            far.append(transform(proj, s).area / 1000000)

    print(
        "\n"
        + "Total estimated cost to quota: "
        + str("{:,}".format(round(sum(far))))
        + " sqkm"
    )
    if not len(ar) == 0:
        print(
            "Total estimated cost to quota if clipped: "
            + str("{:,}".format(round(sum(ar))))
            + " sqkm"
        )


# idc(idlist=r'C:\planet_demo\opencadover.csv',item='PSScene4Band',asset='analytic',geometry=r'C:\Users\samapriya\Downloads\sfo.geojson')



from __future__ import print_function

__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import subprocess
import argparse
import os
import time
import sys
import json
import base64
import requests
import webbrowser
import clipboard
import platform
import datetime
import pkg_resources
from bs4 import BeautifulSoup
from pySmartDL import SmartDL
from os.path import expanduser

if str(platform.system().lower()) == "windows":
    # Get python runtime version
    version = sys.version_info[0]
    try:
        import pipwin

        if pipwin.__version__ == "0.5.0":
            pass
        else:
            a = subprocess.call(
                "{} -m pip install pipwin==0.5.0".format(sys.executable),
                shell=True,
                stdout=subprocess.PIPE,
            )
            b = subprocess.call(
                "{} -m pip install wheel".format(sys.executable),
                shell=True,
                stdout=subprocess.PIPE,
            )
            subprocess.call("pipwin refresh", shell=True)
        """Check if the pipwin cache is old: useful if you are upgrading porder on windows
        [This section looks if the pipwin cache is older than two weeks]
        """
        home_dir = expanduser("~")
        fullpath = os.path.join(home_dir, ".pipwin")
        file_mod_time = os.stat(fullpath).st_mtime
        if int((time.time() - file_mod_time) / 60) > 43200:
            print("Refreshing your pipwin cache")
            subprocess.call("pipwin refresh", shell=True)
    except ImportError:
        a = subprocess.call(
            "{} -m pip install pipwin==0.5.0".format(sys.executable),
            shell=True,
            stdout=subprocess.PIPE,
        )
        subprocess.call("pipwin refresh", shell=True)
    except Exception as e:
        print(e)
    try:
        import gdal
    except ImportError:
        subprocess.call("pipwin install gdal", shell=True)
    except Exception as e:
        print(e)
    try:
        import pyproj
    except ImportError:
        subprocess.call("pipwin install pyproj", shell=True)
    except Exception as e:
        print(e)
    try:
        import shapely
    except ImportError:
        subprocess.call("pipwin install shapely", shell=True)
    except Exception as e:
        print(e)
    try:
        import fiona
    except ImportError:
        subprocess.call("pipwin install fiona", shell=True)
    except Exception as e:
        print(e)
    try:
        import geopandas
    except ImportError:
        subprocess.call("pipwin install geopandas", shell=True)
    except Exception as e:
        print(e)
from .conv2geojson import convert
from .geojson_simplify import geosimple
from .geojson2id import idl
from .text_split import idsplit
from .order_now import order
from .order_size import ordersize
from .downloader import download
from .diffcheck import checker
from .ordstat import ostat
from .async_downloader import asyncdownload
from .idcheck import idc
from .resubmit import reorder
from planet.api.auth import find_api_key

if str(platform.python_version()) > "3.3.0":
    from .async_down import downloader
os.chdir(os.path.dirname(os.path.realpath(__file__)))
lpath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(lpath)


# Get package version
def porder_version():
    url = "https://pypi.org/project/porder/"
    source = requests.get(url)
    html_content = source.text
    soup = BeautifulSoup(html_content, "html.parser")
    company = soup.find("h1")
    if (
        not pkg_resources.get_distribution("porder").version
        == company.string.strip().split(" ")[-1]
    ):
        print(
            "\n"
            + "========================================================================="
        )
        print(
            "Current version of porder is {} upgrade to lastest version: {}".format(
                pkg_resources.get_distribution("porder").version,
                company.string.strip().split(" ")[-1],
            )
        )
        print(
            "========================================================================="
        )


def version_from_parser(args):
    porder_version()


# Go to the readMe
def readme():
    try:
        a = webbrowser.open("https://tyson-swetnam.github.io/porder/", new=2)
        if a == False:
            print("Your setup does not have a monitor to display the webpage")
    except Exception as e:
        print(e)


def read_from_parser(args):
    readme()


# Function to get user's quota
def planet_quota():
    # Get API Key: Requires user to have initialized Planet CLI
    try:
        api_key = find_api_key()
        os.environ["PLANET_API_KEY"] = find_api_key()
    except Exception as e:
        print("Failed to get Planet Key: Try planet init")
        sys.exit()
    """Print allocation and remaining quota in Sqkm."""
    try:
        main = requests.get(
            "https://api.planet.com/auth/v1/experimental/public/my/subscriptions",
            auth=(api_key, ""),
        )
        if main.status_code == 200:
            content = main.json()
            for item_id in content:
                try:
                    print(" ")
                    print("Subscription ID: %s" % item_id["id"])
                    print("Plan ID: %s" % item_id["plan_id"])
                    print("Allocation Name: %s" % item_id["organization"]["name"])
                    print(
                        "Allocation active from: %s"
                        % item_id["active_from"].split("T")[0]
                    )
                    if item_id["active_to"] is not None:
                        print(
                            "Allocation active to: %s"
                            % item_id["active_to"].split("T")[0]
                        )
                    print("Quota Enabled: %s" % item_id["quota_enabled"])
                    print("Total Quota in SqKm: %s" % item_id["quota_sqkm"])
                    print("Total Quota used: %s" % item_id["quota_used"])
                    if (item_id["quota_sqkm"]) is not None:
                        leftquota = float(
                            item_id["quota_sqkm"] - float(item_id["quota_used"])
                        )
                        print("Remaining Quota in SqKm: %s" % leftquota)
                    else:
                        print("No Quota Allocated")
                    print("")
                except Exception as e:
                    pass
        elif main.status_code == 500:
            print("Temporary issue: Try again")
        else:
            print("Failed with exception code: " + str(main.status_code))

    except IOError:
        print("Initialize client or provide API Key")


def planet_quota_from_parser(args):
    planet_quota()


# Convert folder with multiple shapefiles to geojsons
def convert_metadata_from_parser(args):
    convert(folder=args.source, export=args.destination)


# base64 encoding for GCS credentials
def gcs_cred(cred):
    with open(cred) as file:
        aoi_resp = json.load(file)
        filestr = json.dumps(aoi_resp)
        print("")
        bstr = filestr.encode("utf-8")
        try:
            clipboard.copy(base64.b64encode(bstr).decode("ascii"))
            print("base64 encoding copied to clipboard")
        except Exception as e:
            print("Unable to copy to clipboard" + "\n")
            print(base64.b64encode(bstr).decode("ascii"))


def gcs_cred_from_parser(args):
    gcs_cred(cred=args.cred)


# Simplify geojson by vertex count
def simplify_from_parser(args):
    geosimple(inp=args.input, output=args.output, num=args.number)


# Create ID List with structured JSON
def idlist_from_parser(args):
    print("")
    if args.asset is None:
        with open(os.path.join(lpath, "bundles.json")) as f:
            r = json.load(f)
            for key, value in r["bundles"].items():
                mydict = r["bundles"][key]["assets"]
                for item_types in mydict:
                    if args.item == item_types:
                        print(
                            "Assets for item "
                            + str(args.item)
                            + " of Bundle type "
                            + str(key)
                            + ": "
                            + str(", ".join(mydict[args.item]))
                        )
        sys.exit()
    idl(
        infile=args.input,
        start=args.start,
        end=args.end,
        item=args.item,
        asset=args.asset,
        num=args.number,
        cmin=args.cmin,
        cmax=args.cmax,
        ovp=args.overlap,
        outfile=args.outfile,
        filters=args.filters,
    )


# Convert folder with multiple shapefiles to geojsons
def idcheck_from_parser(args):
    idc(idlist=args.idlist, item=args.item, asset=args.asset, geometry=args.geometry)


# Check difference from local filelist
def difflist_from_parser(args):
    checker(
        folder=args.folder,
        typ=args.typ,
        infile=args.input,
        item=args.item,
        asset=args.asset,
        start=args.start,
        end=args.end,
        cmin=args.cmin,
        cmax=args.cmax,
        outfile=args.outfile,
    )


# Offcourse all good merge sometimes needs a split
def idsplit_from_parser(args):
    idsplit(infile=args.idlist, linenum=args.lines, output=args.local)


# Get package version
def bundles(item):
    url = "https://developers.planet.com/docs/orders/product-bundles-reference/"
    html = requests.get(url).text
    soup = BeautifulSoup(html, "html.parser")
    with open(os.path.join(lpath, "bundles.json")) as f:
        r = json.load(f)
        current = r["version"]
        curr_date = datetime.datetime.strptime(current, "%Y-%m-%d")
    for detail in soup.find_all("em"):
        release = detail.text.split(":")[1].strip()
        ref_date = datetime.datetime.strptime(release, "%Y-%m-%d")
    if curr_date < ref_date:
        print("Refreshing bundles to " + "\n" + str(release))
        try:
            url = "https://developers.planet.com/theme/js/scrub_all_bundles_bundle_spec.json"
            dest = os.path.join(lpath, "bundles.json")
            obj = SmartDL(url, dest)
            obj.start()
            path = obj.get_dest()
        except Exception as e:
            print(e)
    else:
        print("Bundles version: " + str(current) + "\n")
    print("")
    with open(os.path.join(lpath, "bundles.json")) as f:
        r = json.load(f)
        for key, value in r["bundles"].items():
            mydict = r["bundles"][key]["assets"]
            for item_types in mydict:
                if item == item_types:
                    print(
                        "Bundle type: "
                        + str(key)
                        + "\n"
                        + str(", ".join(mydict[item]))
                        + "\n"
                    )


def bundles_from_parser(args):
    bundles(item=args.item)


# Place the order
def order_from_parser(args):
    order(
        name=args.name,
        idlist=args.idlist,
        item=args.item,
        asset=args.bundle,
        sid=args.sid,
        op=args.op,
        anchor = args.anchor,
        format = args.format,
        boundary=args.boundary,
        projection=args.projection,
        gee=args.gee,
        kernel=args.kernel,
        compression=args.compression,
        aws=args.aws,
        azure=args.azure,
        gcs=args.gcs,
    )


# Reorer an old or failed order
def reorder_from_parser(args):
    reorder(url=args.url, notification=args.notification)


# Cancel an order or all order
def cancel(id):
    headers = {"Content-Type": "application/json"}
    # Get API Key: Requires user to have initialized Planet CLI
    try:
        api_key = find_api_key()
    except Exception as e:
        print("Failed to get Planet Key: Try planet init")
        sys.exit()
    if id == "all":
        url = "https://api.planet.com/compute/ops/bulk/orders/v2/cancel"
        resp = requests.post(url, data="{}", headers=headers, auth=(api_key, ""))
        if resp.status_code == 200:
            print(
                "Number of orders failed to cancel: "
                + str(resp.json()["result"]["failed"]["count"])
            )
            print(
                "Number of orders successfully cancelled: "
                + str(resp.json()["result"]["succeeded"]["count"])
            )
        else:
            print(
                "Failed wth status code & error: "
                + str(resp.status_code)
                + " : "
                + str(resp.json())
            )
    else:
        url = "https://api.planet.com/compute/ops/orders/v2/" + str(id)
        resp = requests.put(url, headers=headers, auth=(api_key, ""))
        if resp.status_code == 200:
            print("Orders ID " + str(id) + " successfully cancelled")
        else:
            print(
                "Failed wth status code & error: "
                + str(resp.status_code)
                + " : "
                + str(resp.json())
            )


def cancel_from_parser(args):
    cancel(id=args.id)


# Get size of order in human size
def ordersize_from_parser(args):
    ordersize(url=args.url)


# Get concurrent orders that are running
def stats():
    # Get API Key: Requires user to have initialized Planet CLI
    try:
        api_key = find_api_key()
        os.environ["PLANET_API_KEY"] = find_api_key()
    except Exception as e:
        print("Failed to get Planet Key: Try planet init")
        sys.exit()

    SESSION = requests.Session()
    SESSION.auth = (api_key, "")
    print("Checking on all running orders...")
    result = SESSION.get("https://api.planet.com/compute/ops/stats/orders/v2")
    if int(result.status_code) == 200:
        page = result.json()
        try:
            print(
                "\n"
                + "Total queued order for organization: "
                + str(page["organization"]["queued_orders"])
            )
            print(
                "Total running orders for organization: "
                + str(page["organization"]["running_orders"])
            )
            print(
                "\n"
                + "Total queued orders for user: "
                + str(page["user"]["queued_orders"])
            )
            print(
                "Total running orders for user: " + str(page["user"]["running_orders"])
            )
        except Exception as e:
            print(e)
    elif int(result.status_code) == 401:
        print("Access denied - insufficient privileges")
    elif int(result.status_code) == 500:
        print("Server Error")
    else:
        print("Failed with " + str(result.status_code) + " " + str(result.text))


def stats_from_parser(args):
    stats()


# Get order list by state and date
def ostate_from_parser(args):
    ostat(state=args.state, start=args.start, end=args.end, limit=args.limit)


# Download the order
def download_from_parser(args):
    download(url=args.url, local=args.local, ext=args.ext)


# Multithreaded downloader
def asyncdownload_from_parser(args):
    asyncdownload(url=args.url, local=args.local, ext=args.ext)


def multiproc_from_parser(args):
    if str(platform.python_version()) > "3.3.0":
        downloader(url=args.url, final=args.local, ext=args.ext)
    elif str(platform.python_version()) <= "3.3.0":
        if args.ext == None:
            subprocess.call(
                "python multiproc_pydl.py " + args.url + " " + args.local + " ",
                shell=True,
            )
        else:
            subprocess.call(
                "python multiproc_pydl.py "
                + args.url
                + " "
                + args.local
                + " "
                + args.ext,
                shell=True,
            )


porder_version()
print("")

spacing = "                               "


def main(args=None):
    parser = argparse.ArgumentParser(description="Ordersv2 Simple Client")
    subparsers = parser.add_subparsers()

    parser_read = subparsers.add_parser(
        "readme", help="Go the web based porder readme page"
    )
    parser_read.set_defaults(func=read_from_parser)

    parser_planet_quota = subparsers.add_parser(
        "quota", help="Prints your Planet Quota Details"
    )
    parser_planet_quota.set_defaults(func=planet_quota_from_parser)

    parser_convert = subparsers.add_parser(
        "convert", help="Convert all shapefiles or kmls in folder to GeoJSON"
    )
    parser_convert.add_argument("--source", help="Choose Source Folder")
    parser_convert.add_argument("--destination", help="Choose Destination Folder")
    parser_convert.set_defaults(func=convert_metadata_from_parser)

    parser_gcs_cred = subparsers.add_parser("base64", help="Base 64 encode a JSON file")
    required_named = parser_gcs_cred.add_argument_group("Required named arguments.")
    required_named.add_argument(
        "--cred", help="Path to GCS credential file", required=True
    )
    parser_gcs_cred.set_defaults(func=gcs_cred_from_parser)

    parser_simplify = subparsers.add_parser(
        "simplify",
        help="Simplifies geometry to number of vertices specified using Visvalingam-Wyatt line simplification algorithm",
    )
    parser_simplify.add_argument("--input", help="Input GeoJSON file")
    parser_simplify.add_argument("--output", help="Output simplified GeoJSON file")
    parser_simplify.add_argument(
        "--number", help="Total number of vertices in output GeoJSON"
    )
    parser_simplify.set_defaults(func=simplify_from_parser)

    parser_idlist = subparsers.add_parser(
        "idlist", help="Get idlist using geometry & filters"
    )
    required_named = parser_idlist.add_argument_group("Required named arguments.")
    required_named.add_argument(
        "--input", help="Input geometry file for now geojson/json/kml", required=True
    )
    required_named.add_argument(
        "--start", help="Start Date &/or Time yyyy-mm-ddTHH:MM:SS", required=True
    )
    required_named.add_argument(
        "--end", help="End Date &/or Time yyyy-mm-ddTHH:MM:SS", required=True
    )
    required_named.add_argument(
        "--item",
        help="Item Type PSScene4Band|PSOrthoTile|REOrthoTile etc",
        required=True,
    )
    required_named.add_argument(
        "--asset", help="Asset Type analytic, analytic_sr,visual etc", default=None
    )
    required_named.add_argument("--outfile", help="Output csv file", required=True)
    optional_named = parser_idlist.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--cmin", help="Minimum cloud cover 0-1 represents 0 to 100", default=None
    )
    optional_named.add_argument(
        "--cmax", help="Maximum cloud cover 0-1 represents 0 to 100", default=None
    )
    optional_named.add_argument(
        "--number",
        help="Total number of assets, give a large number if you are not sure",
        default=None,
    )
    optional_named.add_argument(
        "--overlap",
        help="Percentage overlap of image with search area range between 0 to 100",
        default=None,
    )
    optional_named.add_argument(
        "--filters",
        nargs="+",
        help="Add an additional string or range filter, Read Me",
        default=None,
    )
    parser_idlist.set_defaults(func=idlist_from_parser)

    parser_difflist = subparsers.add_parser(
        "difflist",
        help="Checks the difference between local files and available Planet assets",
    )
    required_named = parser_difflist.add_argument_group("Required named arguments.")
    required_named.add_argument(
        "--folder",
        help="local folder where image or metadata files are stored",
        required=True,
    )
    required_named.add_argument(
        "--typ", help="File type image or metadata", required=True
    )
    required_named.add_argument(
        "--input", help="Input boundary to search (geojson, json)", required=True
    )
    required_named.add_argument(
        "--item",
        help="Planet Item Type PSScene4Band|PSOrthoTile|REOrthoTile etc",
        required=True,
    )
    required_named.add_argument(
        "--asset", help="Asset Type analytic, analytic_sr,visual etc", required=True
    )
    required_named.add_argument("--start", help="Start date in format YYYY-MM-DD", required=True)
    required_named.add_argument("--end", help="End date in format YYYY-MM-DD", required=True)
    required_named.add_argument(
        "--outfile", help="Full path to CSV file with difference ID list", required=True
    )
    optional_named = parser_difflist.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--cmin", help="Minimum cloud cover 0-1 represents 0 to 100"
    )
    optional_named.add_argument(
        "--cmax", help="Maximum cloud cover 0-1 represents 0 to 100"
    )
    parser_difflist.set_defaults(func=difflist_from_parser)

    parser_idsplit = subparsers.add_parser(
        "idsplit", help="Splits ID list incase you want to run them in small batches"
    )
    parser_idsplit.add_argument("--idlist", help="Idlist txt file to split")
    parser_idsplit.add_argument(
        "--lines", help="Maximum number of lines in each split files"
    )
    parser_idsplit.add_argument(
        "--local", help="Output folder where split files will be exported"
    )
    parser_idsplit.set_defaults(func=idsplit_from_parser)

    parser_idcheck = subparsers.add_parser(
        "idcheck", help="Check idlist for estimating clipped area"
    )
    parser_idcheck.add_argument("--idlist", help="Idlist csv file")
    parser_idcheck.add_argument("--item", help="Item type")
    parser_idcheck.add_argument("--asset", help="Asset type")
    optional_named = parser_idcheck.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--geometry", help="Geometry file for clip if any", default=None
    )
    parser_idcheck.set_defaults(func=idcheck_from_parser)

    parser_bundles = subparsers.add_parser(
        "bundles", help="Check bundles of assets for given tiem type"
    )
    parser_bundles.add_argument("--item", help="Item type")
    parser_bundles.set_defaults(func=bundles_from_parser)

    parser_order = subparsers.add_parser(
        "order",
        help='Place an order & get order url currently supports "toar","clip","composite","reproject","compression"',
    )
    required_named = parser_order.add_argument_group("Required named arguments.")
    required_named.add_argument(
        "--name", help="Order Name to be Submitted", required=True
    )
    required_named.add_argument(
        "--idlist", help="CSV idlist with item IDs", required=True
    )
    required_named.add_argument(
        "--item",
        help="Item Type PSScene4Band|PSOrthoTile|REOrthoTile etc",
        required=True,
    )
    required_named.add_argument(
        "--bundle",
        help="Bundle Type: analytic, analytic_sr,analytic_sr_udm2",
        required=True,
    )
    optional_named = parser_order.add_argument_group("Optional named arguments")
    optional_named.add_argument("--sid", help="Subscription ID", default=None)
    optional_named.add_argument(
        "--boundary",
        help="Boundary/geometry for clip operation geojson|json|kml",
        default=None,
    )
    optional_named.add_argument(
        "--projection",
        help='Projection for reproject operation of type "EPSG:4326"',
        default=None,
    )
    optional_named.add_argument(
        "--gee",
        help='provide gee-project,gee-collection',
        default=None,
    )
    optional_named.add_argument(
        "--kernel",
        help='Resampling kernel used "near", "bilinear", "cubic", "cubicspline", "lanczos", "average" and "mode"',
        default=None,
    )
    optional_named.add_argument(
        "--compression",
        help='Compression type used for tiff_optimize tool, "lzw"|"deflate"',
        default=None,
    )
    optional_named.add_argument(
        "--anchor",
        help='Anchor Item ID to use for the coregistration tool',
        default=None,
    )
    optional_named.add_argument(
        "--format",
        help='Delivery format choose from COG/PL_NITF to use for the format tool',
        default=None,
    )
    optional_named.add_argument(
        "--aws", help="AWS cloud credentials config yml file", default=None
    )
    optional_named.add_argument(
        "--azure", help="Azure cloud credentials config yml file", default=None
    )
    optional_named.add_argument(
        "--gcs", help="GCS cloud credentials config yml file", default=None
    )
    optional_named.add_argument(
        "--op",
        nargs="+",
        help="Add operations, delivery & notification clip|toar|harmonize|composite|zip|zipall|compression|projection|kernel|coreg|format|aws|azure|gcs|gee|email <Choose indices from>: ndvi|gndvi|bndvi|ndwi|tvi|osavi|evi2|msavi2|sr",
        default=None,
    )

    parser_order.set_defaults(func=order_from_parser)

    parser_reorder = subparsers.add_parser("reorder", help="Reorder an existing order")
    required_named = parser_reorder.add_argument_group("Required named arguments.")
    required_named.add_argument("--url", help="Order url to be ordered", required=True)
    optional_named = parser_reorder.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--notification", help='Use "email" to get an email notification', default=None
    )
    parser_reorder.set_defaults(func=reorder_from_parser)

    parser_cancel = subparsers.add_parser("cancel", help="Cancel queued order(s)")
    parser_cancel.add_argument(
        "--id", help='order id you want to cancel use "all" to cancel all'
    )
    parser_cancel.set_defaults(func=cancel_from_parser)

    parser_ordersize = subparsers.add_parser(
        "ordersize", help="Estimate total download size"
    )
    parser_ordersize.add_argument("--url", help="order url you got for your order")
    parser_ordersize.set_defaults(func=ordersize_from_parser)

    parser_ostate = subparsers.add_parser(
        "ostate", help="Get list of orders by state and date range"
    )
    parser_ostate.add_argument(
        "--state",
        help="choose state between queued| running | success | failed | partial",
    )
    parser_ostate.add_argument("--start", help="start date in format YYYY-MM-DD")
    parser_ostate.add_argument("--end", help="end date in format YYYY-MM-DD")
    optional_named = parser_ostate.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--limit", help="Limit the maximum table size", default=None
    )
    parser_ostate.set_defaults(func=ostate_from_parser)

    parser_stats = subparsers.add_parser(
        "stats", help="Prints number of orders queued and running for org & user"
    )
    parser_stats.set_defaults(func=stats_from_parser)

    parser_download = subparsers.add_parser(
        "download", help="Downloads all files in your order"
    )
    parser_download.add_argument("--url", help="order url you got for your order")
    parser_download.add_argument(
        "--local", help="Output folder where ordered files will be exported"
    )
    optional_named = parser_download.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--ext", help="File Extension to download", default=None
    )
    parser_download.set_defaults(func=download_from_parser)

    parser_asyncdownload = subparsers.add_parser(
        "multipart", help="Uses multiprocessing to download for all files in your order"
    )
    parser_asyncdownload.add_argument("--url", help="order url you got for your order")
    parser_asyncdownload.add_argument(
        "--local", help="Output folder where ordered files will be exported"
    )
    optional_named = parser_asyncdownload.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--ext", help="File Extension to download", default=None
    )
    parser_asyncdownload.set_defaults(func=asyncdownload_from_parser)

    parser_multiproc = subparsers.add_parser(
        "multiproc",
        help="Multiprocess based downloader to download for all files in your order",
    )
    parser_multiproc.add_argument("--url", help="order url you got for your order")
    parser_multiproc.add_argument(
        "--local", help="Output folder where ordered files will be exported"
    )
    optional_named = parser_multiproc.add_argument_group("Optional named arguments")
    optional_named.add_argument(
        "--ext", help="File Extension to download", default=None
    )
    parser_multiproc.set_defaults(func=multiproc_from_parser)
    args = parser.parse_args()

    try:
        func = args.func
    except AttributeError:
        parser.error("too few arguments")
    func(args)


if __name__ == "__main__":
    main()



from __future__ import print_function

__copyright__ = """
    Copyright 2019 Samapriya Roy
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
       http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__license__ = "Apache 2.0"

import requests
import asyncio
import os
import json
import glob
import progressbar
import sys
import time
from concurrent.futures import ThreadPoolExecutor
from timeit import default_timer
from retrying import retry
from planet.api.auth import find_api_key

# Get Planet API and Authenticate SESSION
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")

suffixes = ["B", "KB", "MB", "GB", "TB", "PB"]


def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    f = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (f, suffixes[i])


@retry(wait_exponential_multiplier=1000, wait_exponential_max=10000)
def check_for_redirects(url):
    try:
        r = SESSION.get(url, allow_redirects=False, timeout=0.5)
        if 300 <= r.status_code < 400:
            return r.headers["location"]
        elif r.status_code == 429:
            raise Exception("rate limit error")
    except requests.exceptions.Timeout:
        return "[timeout]"
    except requests.exceptions.ConnectionError:
        return "[connection error]"
    except requests.HTTPError as e:
        print(r.status_code)
        if r.status_code == 429:  # Too many requests
            raise Exception("rate limit error")


START_TIME = default_timer()


def fetch(session, url):
    urlcheck = url.split("|")[0]
    fullpath = url.split("|")[1]
    [head, tail] = os.path.split(fullpath)
    # print("Starting download of %s" % fullpath.split('/')[-1])
    if not os.path.exists(head):
        os.makedirs(head)
    os.chdir(head)
    if not os.path.isfile(fullpath):
        r = session.get(urlcheck, stream=True)
        with open(fullpath, "wb") as f:
            for ch in r:
                f.write(ch)
    elapsed = default_timer() - START_TIME
    time_completed_at = "{:5.2f}s".format(elapsed)
    print("{0:100} {1:20}".format(tail, time_completed_at))

    return tail


urls = []
size_list = []


def funct(url, final, ext):
    filenames = glob.glob1(final, "*")
    if not os.path.exists(final):
        os.makedirs(final)
    os.chdir(final)
    response = SESSION.get(url).json()
    print("Polling with exponential backoff..")
    while (
        response["state"] == "queued"
        or response["state"] == "running"
        or response["state"] == "starting"
    ):
        bar = progressbar.ProgressBar()
        for z in bar(range(60)):
            time.sleep(1)
        response = SESSION.get(url).json()
    if response["state"] == "success" or response["state"] == "partial":
        print("Order completed with status: " + str(response["state"]))
        print("")
        for files in response["_links"]["results"]:
            if files["name"].endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(files["location"]).json()
                for things in resp["files"]:
                    size_list.append(things["size"])
        print("Estimated Download Size for order: {}".format(humansize(sum(size_list))))
        print("")
        for items in response["_links"]["results"]:
            url = items["location"]
            name = items["name"]
            if name.endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(url)
                if int(resp.status_code) == 200:
                    r = resp.content
                    inp = json.loads(r)
                    for things in inp["files"]:
                        try:
                            local_path = os.path.join(
                                final, things["path"].split("/")[-1]
                            )
                        except Exception as e:
                            print(e)
                else:
                    print(resp.status_code)
            else:
                local_path = os.path.join(final, str(os.path.split(items["name"])[-1]))
            filenames = [os.path.join(final, files) for files in filenames]
            if not local_path in filenames:
                url_to_check = url if url.startswith("https") else "http://%s" % url
                redirect_url = check_for_redirects(url_to_check)
                if not os.path.isfile(local_path) and ext is None:
                    urls.append(str(redirect_url) + "|" + local_path)
                    print("Processing total URLs " + str(len(urls)), end="\r")
                if not os.path.isfile(local_path) and ext is not None:
                    if local_path.endswith(ext):
                        urls.append(str(redirect_url) + "|" + local_path)
                        print("Processing total URLs " + str(len(urls)), end="\r")
    else:
        print("Order Failed with state: " + str(response["state"]))
    print("Processing a url list with " + str(len(urls)) + " items")
    print("\n")
    return urls


async def get_data_asynchronous(url, final, ext):
    urllist = funct(url=url, final=final, ext=ext)
    print("{0:100} {1:20}".format("File", "Completed at"))
    with ThreadPoolExecutor(max_workers=10) as executor:
        with requests.Session() as session:
            # Set any session parameters here before calling `fetch`
            loop = asyncio.get_event_loop()
            START_TIME = default_timer()
            tasks = [
                loop.run_in_executor(
                    executor,
                    fetch,
                    *(
                        session,
                        url,
                    )  # Allows us to pass in multiple arguments to `fetch`
                )
                for url in urllist
            ]
            for response in await asyncio.gather(*tasks):
                pass


def downloader(url, final, ext):
    loop = asyncio.get_event_loop()
    future = asyncio.ensure_future(get_data_asynchronous(url, final, ext))
    loop.run_until_complete(future)


# downloader(url='https://api.planet.com/compute/ops/orders/v2/bbccc868-bada-4a4c-8c1d-9d8ef81c1d75',final=r'C:\planet_demo\mp2',ext=None)



__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import requests
import time
import progressbar
import json
import os
import sys
import csv
import glob
from retrying import retry
from pySmartDL import SmartDL
from planet.api.utils import read_planet_json
from planet.api.auth import find_api_key
from datetime import datetime


# Get Planet API and Authenticate SESSION
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")


suffixes = ["B", "KB", "MB", "GB", "TB", "PB"]


def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    f = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (f, suffixes[i])


i = 1


@retry(wait_exponential_multiplier=1000, wait_exponential_max=10000)
def check_for_redirects(url):
    try:
        r = SESSION.get(url, allow_redirects=False, timeout=0.5)
        if 300 <= r.status_code < 400:
            return r.headers["location"]
        elif r.status_code == 429:
            raise Exception("rate limit error")
    except requests.exceptions.Timeout:
        return "[timeout]"
    except requests.exceptions.ConnectionError:
        return "[connection error]"
    except requests.HTTPError as e:
        if r.status_code == 429:  # Too many requests
            raise Exception("rate limit error")


@retry(wait_exponential_multiplier=1000, wait_exponential_max=10000)
def downonly(redirect_url, local_path, ext, items, ulength):
    global i
    result = SESSION.get(redirect_url)
    if not os.path.exists(local_path) and result.status_code == 200:
        if ext is not None:
            if local_path.endswith(ext):
                print(
                    str(ulength - i) + " remaining ==> Downloading: " + str(local_path)
                )
                i = i + 1
                obj = SmartDL(redirect_url, local_path)
                obj.start()
                path = obj.get_dest()
        elif ext is None:
            print(str(ulength - i) + " remaining ==> Downloading: " + str(local_path))
            i = i + 1
            obj = SmartDL(redirect_url, local_path)
            obj.start()
            path = obj.get_dest()
    elif result.status_code == 429:
        raise Exception("rate limit error")
    else:
        if int(result.status_code) != 200:
            print(
                "Encountered error with code: "
                + str(result.status_code)
                + " for "
                + str(os.path.split(items["name"])[-1])
            )
        elif int(result.status_code) == 200:
            print(
                "Checking "
                + str(ulength - i)
                + " remaining ==> "
                + "File already exists SKIPPING: "
                + str(os.path.split(local_path)[-1])
            )
            i = i + 1


def get_session_response(url):
    response = SESSION.get(url).json()
    print("Polling ...")
    while (
        response["state"] == "queued"
        or response["state"] == "running"
        or response["state"] == "starting"
    ):
        bar = progressbar.ProgressBar()
        for _ in bar(range(60)):
            time.sleep(1)
        response = SESSION.get(url).json()

    return response


size_list = []
# Get the redirects and download
def asyncdownload(url, local, ext):
    filenames = glob.glob1(local, "*")
    response = get_session_response(url)
    if response["state"] == "success" or response["state"] == "partial":
        print("Order completed with status: " + str(response["state"]))
        ulength = len(response["_links"]["results"]) - len(filenames)
        print("")
        for files in response["_links"]["results"]:
            if files["name"].endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(files["location"]).json()
                for things in resp["files"]:
                    size_list.append(things["size"])
        print("Estimated Download Size for order: {}".format(humansize(sum(size_list))))
        print("")
        for index, items in enumerate(response["_links"]["results"]):
            expiration_time = datetime.strptime(
                items["expires_at"], "%Y-%m-%dT%H:%M:%S.%fZ"
            )
            if datetime.now() > expiration_time:
                print("The URL links have expired.  Refreshing.")
                response[:] = get_session_response(url)
                if response["state"] == "success" or response["state"] == "partial":
                    items = response["_links"]["results"][index]
                else:
                    print("Order Failed with state: " + str(response["state"]))
                    return
            url = items["location"]
            name = items["name"]
            if name.endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(url)
                if int(resp.status_code) == 200:
                    r = resp.content
                    inp = json.loads(r)
                    for things in inp["files"]:
                        try:
                            local_path = os.path.join(
                                local, things["path"].split("/")[-1]
                            )
                        except Exception as e:
                            print(e)
                else:
                    print(resp.status_code)
            else:
                local_path = os.path.join(local, str(os.path.split(items["name"])[-1]))
            filenames = [os.path.join(local, files) for files in filenames]
            if not local_path in filenames:
                url_to_check = url if url.startswith("https") else "http://%s" % url
                redirect_url = check_for_redirects(url_to_check)
                if redirect_url.startswith("https"):
                    try:
                        downonly(redirect_url, local_path, ext, items, ulength)
                    except Exception as e:
                        print(e)
                    except (KeyboardInterrupt, SystemExit) as e:
                        print("\n" + "Program escaped by User")
                        sys.exit()
            else:
                print("File already exists SKIPPING: " + str(local_path))
    else:
        print("Order Failed with state: " + str(response["state"]))


##download(url="https://api.planet.com/compute/ops/orders/v2/0ee9e923-59fc-4c31-8632-9882cb342708",
##         local=r"C:\planet_demo\terra",errorlog=r"C:\planet_demo\errorlog.csv")



import requests
import time
import progressbar
import json
import os
import sys
from retrying import retry
from planet.api.utils import read_planet_json
from planet.api.auth import find_api_key


sz = []
fname = []
suffixes = ["B", "KB", "MB", "GB", "TB", "PB"]


def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    f = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (f, suffixes[i])


# Get Planet API and Authenticate SESSION
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")


def get_session_response(url):
    response = SESSION.get(url).json()
    while (
        response["state"] == "queued"
        or response["state"] == "running"
        or response["state"] == "starting"
    ):
        bar = progressbar.ProgressBar()
        for _ in bar(range(60)):
            time.sleep(1)
        response = SESSION.get(url).json()

    return response


size_list = []


def ordersize(url):
    response = get_session_response(url)
    if response["state"] == "success" or response["state"] == "partial":
        print("")
        print("Order completed with status: " + str(response["state"]))
        for files in response["_links"]["results"]:
            if files["name"].endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(files["location"]).json()
                for things in resp["files"]:
                    size_list.append(things["size"])
        print("Estimated Download Size for order: {}".format(humansize(sum(size_list))))



from __future__ import print_function

__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"


#!/usr/bin/env python

import multiprocessing
import os
import csv
import requests
import glob
import time
import progressbar
import json
import sys
from threading import *

screen_lock = Semaphore(value=1)
from retrying import retry
from planet.api.utils import read_planet_json
from planet.api.auth import find_api_key

# Get Planet API and Authenticate SESSION
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")

# To get redirect link
@retry(wait_exponential_multiplier=1000, wait_exponential_max=10000)
def check_for_redirects(url):
    try:
        r = SESSION.get(url, allow_redirects=False, timeout=0.5)
        if 300 <= r.status_code < 400:
            return r.headers["location"]
        elif r.status_code == 429:
            raise Exception("rate limit error")
    except requests.exceptions.Timeout:
        return "[timeout]"
    except requests.exceptions.ConnectionError:
        return "[connection error]"
    except requests.HTTPError as e:
        print(r.status_code)
        if r.status_code == 429:  # Too many requests
            raise Exception("rate limit error")


########################################################################
class MultiProcDownloader(object):
    """
    Downloads urls with Python's multiprocessing module
    """

    # ----------------------------------------------------------------------
    def __init__(self, urls):
        """ Initialize class with list of urls """
        self.urls = urls

    # ----------------------------------------------------------------------
    def run(self):
        """
        Download the urls and waits for the processes to finish
        """
        jobs = []
        for url in self.urls:
            process = multiprocessing.Process(target=self.worker, args=(url,))
            jobs.append(process)
            process.start()
        for job in jobs:
            job.join()

    # ----------------------------------------------------------------------
    @retry(wait_exponential_multiplier=1000, wait_exponential_max=10000)
    def worker(self, url):
        """
        The target method that the process uses tp download the specified url
        """
        try:
            urlcheck = url.split("|")[0]
            fullpath = url.split("|")[1]
            [head, tail] = os.path.split(fullpath)
            msg = "Starting download of %s" % fullpath.split("/")[-1]
            if not os.path.exists(head):
                os.makedirs(head)
            os.chdir(head)
            if not os.path.isfile(fullpath):
                screen_lock.acquire()
                print(str(msg) + " " + str(multiprocessing.current_process().name))
                screen_lock.release()
                r = requests.get(urlcheck)
                with open(fullpath, "wb") as f:
                    f.write(r.content)
            else:
                print(("File already exists skipping " + str(tail)))
        except Exception as e:
            print(e)
            print(("Issues with file: " + str(fullpath)))


def funct(url, final, ext):
    filenames = glob.glob1(final, "*")
    if not os.path.exists(final):
        os.makedirs(final)
    os.chdir(final)
    urls = []
    response = SESSION.get(url).json()
    print("Polling with exponential backoff..")
    while (
        response["state"] == "queued"
        or response["state"] == "running"
        or response["state"] == "starting"
    ):
        bar = progressbar.ProgressBar()
        for z in bar(range(60)):
            time.sleep(1)
        response = SESSION.get(url).json()
    if response["state"] == "success" or response["state"] == "partial":
        print("Order completed with status: " + str(response["state"]))
        for items in response["_links"]["results"]:
            url = items["location"]
            name = items["name"]
            if name.endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(url)
                if int(resp.status_code) == 200:
                    r = resp.content
                    inp = json.loads(r)
                    for things in inp["files"]:
                        try:
                            local_path = os.path.join(
                                final,
                                things["annotations"]["planet/item_id"]
                                + "_manifest.json",
                            )
                        except Exception as e:
                            local_path = os.path.join(
                                final,
                                things["path"].split("/")[1].split(".")[0]
                                + "_manifest.json",
                            )
                else:
                    print(resp.status_code)
            else:
                local_path = os.path.join(final, str(os.path.split(items["name"])[-1]))
            filenames = [os.path.join(final, files) for files in filenames]
            if not local_path in filenames:
                url_to_check = url if url.startswith("https") else "http://%s" % url
                redirect_url = check_for_redirects(url_to_check)
                if redirect_url.startswith("https"):
                    if not os.path.isfile(local_path) and ext is None:
                        urls.append(str(redirect_url) + "|" + local_path)
                        print("Processing total URLs " + str(len(urls)), end="\r")
                    if not os.path.isfile(local_path) and ext is not None:
                        if local_path.endswith(ext):
                            urls.append(str(redirect_url) + "|" + local_path)
                            print("Processing total URLs " + str(len(urls)), end="\r")
    else:
        print("Order Failed with state: " + str(response["state"]))
    print("Downloading a total of " + str(len(urls)) + " objects")
    downloader = MultiProcDownloader(urls)
    downloader.run()


# ----------------------------------------------------------------------
if __name__ == "__main__":
    if len(sys.argv) == 4:
        funct(url=sys.argv[1], final=os.path.normpath(sys.argv[2]), ext=sys.argv[3])
    elif len(sys.argv) == 3:
        funct(url=sys.argv[1], final=os.path.normpath(sys.argv[2]), ext=None)
    # funct(url='https://api.planet.com/compute/ops/orders/v2/4ebfa89e-dc59-41cc-ad82-dbad2b5375b2',final=r'C:\planet_demo',ext='.zip')
    # downloader = MultiProcDownloader(urls)
    # downloader.run()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import geopandas as gpd
from .kml2ee import kml2coord
import os


def convert(folder, export):
    for items in os.listdir(folder):
        if items.endswith(".shp"):
            inD = gpd.read_file(os.path.join(folder, items), encoding="utf-8")
            # Reproject to EPSG 4326
            try:
                data_proj = inD.copy()
                data_proj["geometry"] = data_proj["geometry"].to_crs(epsg=4326)
                data_proj.to_file(
                    os.path.join(export, str(items).replace(".shp", ".geojson")),
                    driver="GeoJSON",
                )
                print(
                    "Export completed to "
                    + str(os.path.join(export, str(items).replace(".shp", ".geojson")))
                )
            except Exception as e:
                print(e)
        elif items.endswith(".kml"):
            try:
                gj = kml2coord(os.path.join(folder, items))
                with open(
                    os.path.join(export, str(items).replace(".kml", ".geojson")), "w"
                ) as outfile:
                    json.dump(gj, outfile)
                print(
                    "Export completed to "
                    + str(os.path.join(export, str(items).replace(".kml", ".geojson")))
                )
            except Exception as e:
                print(e)


# shp2gj(folder=r"C:\Users\samapriya\Downloads\nexgengrid",export=r"C:\planet_demo")



#!/usr/bin/python
# -*- coding: utf-8 -*-
__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import requests
import json
import os
import glob
import sys
from planet.api.auth import find_api_key

# Get API key and authenticate session
try:
    PL_API_KEY = find_api_key()
except Exception as e:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")

l = []


def handle_page(page, asset):
    try:
        for items in page["features"]:
            for itm in items["_permissions"]:
                if itm.split(":")[0] == "assets." + asset:
                    it = items.get("id")
                    # print(it)
                    l.append(it)
    except Exception as e:
        print(e)


def idl(infile, start, end, item, asset, cmin, cmax):
    if cmin is None:
        cmin = 0
    if cmax is None:
        cmax = 1
    headers = {"Content-Type": "application/json"}

    # Parse Geometry
    try:
        if infile.endswith(".geojson"):
            with open(infile) as aoi:
                aoi_resp = json.load(aoi)
                aoi_geom = aoi_resp["features"][0]["geometry"]["coordinates"]
        elif infile.endswith(".json"):
            with open(infile) as aoi:
                aoi_resp = json.load(aoi)
                aoi_geom = aoi_resp["config"][0]["config"]["coordinates"]
    except Exception as e:
        print("Could not parse geometry")
        print(e)
    # Null payload structure
    data = {
        "filter": {
            "type": "AndFilter",
            "config": [
                {
                    "type": "GeometryFilter",
                    "field_name": "geometry",
                    "config": {"type": "Polygon", "coordinates": []},
                },
                {
                    "type": "OrFilter",
                    "config": [
                        {
                            "type": "AndFilter",
                            "config": [
                                {
                                    "type": "StringInFilter",
                                    "field_name": "item_type",
                                    "config": [],
                                },
                                {
                                    "type": "RangeFilter",
                                    "field_name": "cloud_cover",
                                    "config": {"gte": [], "lte": []},
                                },
                                {
                                    "type": "RangeFilter",
                                    "field_name": "sun_elevation",
                                    "config": {"gte": 0, "lte": 90},
                                },
                            ],
                        }
                    ],
                },
                {
                    "type": "OrFilter",
                    "config": [
                        {
                            "type": "DateRangeFilter",
                            "field_name": "acquired",
                            "config": {"gte": [], "lte": []},
                        }
                    ],
                },
            ],
        },
        "item_types": [],
    }
    # Configure search payload
    data["filter"]["config"][0]["config"]["coordinates"] = aoi_geom
    data["filter"]["config"][2]["config"][0]["config"]["gte"] = (
        str(start) + "T04:00:00.000Z"
    )
    data["filter"]["config"][2]["config"][0]["config"]["lte"] = (
        str(end) + "T03:59:59.999Z"
    )
    data["filter"]["config"][1]["config"][0]["config"][1]["config"]["gte"] = float(cmin)
    data["filter"]["config"][1]["config"][0]["config"][1]["config"]["lte"] = float(cmax)
    data["filter"]["config"][1]["config"][0]["config"][0]["config"] = [item]
    data["item_types"] = [item]
    data = str(data).replace("'", '"')

    # Send post request
    result = SESSION.post(
        "https://api.planet.com/data/v1/quick-search", headers=headers, data=data
    )
    if result.status_code == 408:
        sys.exit("Unexpected 408 Error might be intermittent try again in sometime")
    page = result.json()
    final_list = handle_page(page, asset)
    while page["_links"].get("_next") is not None:
        page_url = page["_links"].get("_next")
        page = SESSION.get(page_url).json()
        ids = handle_page(page, asset)
    print("Searched total IDs: " + str(len(l)))
    return set(l)


def checker(folder, typ, infile, item, asset, start, end, cmin, cmax, outfile):
    print("Now Searching....")
    allasset = idl(
        infile=infile,
        item=item,
        asset=asset,
        start=start,
        end=end,
        cmin=cmin,
        cmax=cmax,
    )
    sprefix = {
        "PSScene4Band": "_3B",
        "REOrthoTile": "_R",
        "PSScene3Band": "_3B",
        "PSOrthoTile": "_BGRN",
    }
    sval = sprefix.get(item)
    if typ == "image":
        filenames = glob.glob1(folder, "*.tif")
        l = []
        for items in filenames:
            l.append(items.split(sval)[0])
        print("Number of items not found locally: " + str(len(set(allasset) - set(l))))
        print(
            "IDlist written to "
            + str(outfile)
            + " with "
            + str(len(set(allasset) - set(l)))
        )
        with open(outfile, "w") as f:
            for s in list(set(allasset) - set(l)):
                f.write(str(s) + "\n")
    if typ == "metadata":
        filenames = glob.glob1(folder, "*.xml")
        l = []
        for items in filenames:
            l.append(items.split(sval)[0])
        print(
            "Number of items not found locally: "
            + str(len(set(allasset).difference(set(l))))
        )
        print(
            "IDlist written to "
            + str(outfile)
            + " with "
            + str(len(set(allasset) - set(l)))
            + " ids"
        )
        with open(outfile, "w") as f:
            for s in list(set(allasset) - set(l)):
                f.write(str(s) + "\n")


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="Checks the difference between local files and all available Planet assets"
    )
    parser.add_argument(
        "--folder",
        help="local folder where image or metadata files are stored",
        required=True,
    )
    parser.add_argument("--typ", help="File type image or metadata", required=True)
    parser.add_argument(
        "--input", help="Input boundary to search (geojson, json)", required=True
    )
    parser.add_argument("--item", help="Planet Item Type", required=True)
    parser.add_argument("--asset", help="Planet asset Type", required=True)
    parser.add_argument("--start", help="Start Date YYYY-MM-DD", required=True)
    parser.add_argument("--end", help="End Date YYYY-MM-DD", required=True)
    parser.add_argument(
        "--cmin", help="Minimum cloud cover", required=False, default=None
    )
    parser.add_argument(
        "--cmax", help="Maximum cloud cover", required=False, default=None
    )
    parser.add_argument(
        "--outfile",
        help="Full path to text file with difference ID list",
        required=True,
    )

    args = parser.parse_args()

    checker(
        folder=args.folder,
        typ=args.typ,
        infile=args.input,
        item=args.item,
        asset=args.asset,
        start=args.start,
        end=args.end,
        cmin=args.cmin,
        cmax=args.cmax,
        outfile=args.outfile,
    )
# checker(folder=r'C:\Users\samapriya\Box Sync\IUB\Pycodes\Applications and Tools\Planet Tools\DiffCheck\ps4bxml',typ='metadata',
#     infile=r'C:\Users\samapriya\Box Sync\IUB\Pycodes\Applications and Tools\Planet Tools\DiffCheck\geojson\Grid_3.geojson',
#     item='PSScene4Band',asset='analytic',start='2018-01-01',end='2018-08-01',cmin=None,cmax=0.3,outfile='readme.txt')



__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

#!/usr/bin/python
# -*- coding: utf-8 -*-

import requests
import time
import sys
from datetime import datetime
from datetimerange import DateTimeRange
from planet.api.auth import find_api_key
from prettytable import PrettyTable

x = PrettyTable()

try:
    PL_API_KEY = find_api_key()
except Exception as e:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")


def handle_page(page, start, end):
    for things in page["orders"]:
        s = datetime.strptime(things["created_on"].split("T")[0], "%Y-%m-%d")
        if s in DateTimeRange(start, end):
            try:
                x.field_names = ["name", "items", "url", "created_on"]
                x.add_row(
                    [
                        things["name"],
                        len(things["products"][0]["item_ids"]),
                        things["_links"]["_self"],
                        things["created_on"].split("T")[0],
                    ]
                )
            except Exception as e:
                print(e)


def ostat(state, start, end, limit):
    start = datetime.strptime(start, "%Y-%m-%d")
    end = datetime.strptime(end, "%Y-%m-%d")
    mpage = "https://api.planet.com/compute/ops/orders/v2?state=" + str(state)
    result = SESSION.get(mpage)
    if result.status_code == 200:
        page = result.json()
        final_list = handle_page(page, start, end)
        while page["_links"].get("next") is not None:
            page_url = page["_links"].get("next")
            result = SESSION.get(page_url)
            if result.status_code == 200:
                page = result.json()
                ids = handle_page(page, start, end)
            elif result.status_code == 429:
                time.sleep(1)
                result = SESSION.get(page_url)
                page = result.json()
                ids = handle_page(page, start, end)
            else:
                print(result.status_code)
    if limit is not None:
        print(x.get_string(start=0, end=int(limit)))
    else:
        print(x)


# idl(state="success", start="2019-08-02", end="2019-08-02")



import visvalingamwyatt as vw
import json


# get coordinates list depth
def list_depth(dic, level=1):
    counter = 0
    str_dic = str(dic)
    if "[[[[" in str_dic:
        counter += 1
    return counter


def vertexcount(input):
    with open(input) as aoi:
        aoi_resp = json.load(aoi)
        if list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 0:
            print(
                "Number of vertices in simplified geometry: "
                + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0]))
            )
        elif list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 1:
            print(
                "Number of vertices in simplfied geometry: "
                + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0][0]))
            )
        else:
            print("Please check GeoJSON: Could not parse coordinates")


def geosimple(inp, output, num):
    if int(num) >= 500:
        print("\n" + "Note: Ordersv2 will only accept upto 500 vertex points")
    print("")
    try:
        import fiona

        shape = fiona.open(inp)
        with open(inp) as aoi:
            aoi_resp = json.load(aoi)
            if list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 0:
                print(
                    "Number of current vertices: "
                    + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0]))
                )
            elif list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 1:
                print(
                    "Number of current vertices: "
                    + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0][0]))
                )
            else:
                print("Please check GeoJSON: Could not parse coordinates")
        with fiona.Env():
            with fiona.open(inp, "r") as src:
                with fiona.open(
                    output, "w", schema=src.schema, driver=src.driver, crs=src.crs
                ) as sink:
                    for f in src:
                        if int(num) == 500:
                            num = int(num) - 2
                        else:
                            num = int(num) - 1
                        sink.write(vw.simplify_feature(f, number=num))
            print("Write Completed to: " + str(output))
            vertexcount(output)
    except ImportError:
        with open(inp) as aoi:
            aoi_resp = json.load(aoi)
            if list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 0:
                aoi_geom = aoi_resp["features"][0]["geometry"]["coordinates"]
                print(
                    "Number of current vertices: "
                    + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0]))
                )
            elif list_depth(aoi_resp["features"][0]["geometry"]["coordinates"]) == 1:
                aoi_geom = aoi_resp["features"][0]["geometry"]["coordinates"][0]
                print(
                    "Number of current vertices: "
                    + str(len(aoi_resp["features"][0]["geometry"]["coordinates"][0][0]))
                )
            else:
                print("Please check GeoJSON: Could not parse coordinates")
        ft = {
            "type": "Feature",
            "geometry": {"type": "Polygon", "coordinates": []},
            "properties": {"vertex_count": num},
        }
        ft["geometry"]["coordinates"] = aoi_geom
        # returns a copy of the feature, simplified (using number of vertices)
        if int(num) == 500:
            num = int(num) - 2
        else:
            num = int(num) - 1
        b = vw.simplify_feature(ft, number=num)
        ft["geometry"]["coordinates"] = b["geometry"]["coordinates"]
        with open(output, "w") as g:
            json.dump(ft, g)
        print("Write Completed to: " + str(output))
        vertexcount(output)
    except Exception as e:
        print(e)


# geomsimple(inp=r'C:\Users\samapriya\Downloads\vertex.geojson',output=r'C:\Users\samapriya\Downloads\v2.geojson',num=5)



__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import csv
import sys
import json
import yaml
import requests
import shapely
import clipboard
import visvalingamwyatt as vw
from datetime import date
from shapely.geometry import shape
from prettytable import PrettyTable
from planet.api.utils import read_planet_json
from planet.api.auth import find_api_key


x = PrettyTable()

ovall = []

# get coordinates list depth
def list_depth(dic, level=1):
    counter = 0
    str_dic = str(dic)
    if "[[[[[" in str_dic:
        counter += 2
    elif "[[[[" in str_dic:
        counter += 1
    elif "[[[" in str_dic:
        counter += 0
    return counter


##Setup for bundles
dbundle = {
    "name": [],
    "subscription_id": [],
    "order_type": "partial",
    "products": [{"item_ids": [], "item_type": [], "product_bundle": []}],
    "tools": [],
    "delivery": {},
}
dclip = {"clip": {"aoi": {"type": "MultiPolygon", "coordinates": []}}}
dtoar = {"toar": {"scale_factor": 10000}}
dzip = {"delivery": {"archive_filename": "{{name}}.zip", "archive_type": "zip"}}
dcoreg = {"coregister": {"anchor_item": []}}
dformat = {"file_format": {"format": []}}
dcomposite = {"composite": {}}
dharmonize = {"harmonize": {"target_sensor": "PS2"}}
dreproject = {"reproject": {"projection": [], "kernel": []}}
dtiff = {"tiff_optimize": {"compression": []}}
demail = {"notifications": {"email": True}}
daws = {
    "delivery": {
        "amazon_s3": {
            "bucket": [],
            "aws_region": [],
            "aws_access_key_id": [],
            "aws_secret_access_key": [],
            "path_prefix": [],
        }
    }
}
dazure = {
    "delivery": {
        "azure_blob_storage": {
            "account": [],
            "container": [],
            "sas_token": [],
            "storage_endpoint_suffix": [],
            "path_prefix": [],
        }
    }
}
dgcs = {
    "delivery": {
        "google_cloud_storage": {"bucket": [], "credentials": [], "path_prefix": []}
    }
}
dgee = {
    "delivery": {
        "google_earth_engine": {
            "project": [],
            "collection": []
        }
    }
}

dbmath = {"bandmath": {}}
dszip = {
    "delivery": {
        "archive_filename": "{{name}}_{{order_id}}.zip",
        "archive_type": "zip",
        "single_archive": True,
    }
}
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
url = "https://api.planet.com/compute/ops/orders/v2"


def multipoly(poly):
    multipoly_empty = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"id": 0},
                "geometry": {"type": "MultiPolygon", "coordinates": []},
            }
        ],
    }
    poly_list = []
    with open(poly) as jsonfile:
        data = json.load(jsonfile)
        if len(data["features"]) > 1:
            for things in data["features"]:
                poly_list.append(things["geometry"]["coordinates"])
            multipoly_empty["features"][0]["geometry"]["coordinates"] = poly_list
            return json.dumps(multipoly_empty)
        elif len(data["features"]) == 1:
            return data


def vdepth(dic, level=1):
    counter = 0
    str_dic = str(dic)
    if "[[[[" in str_dic:
        counter += 1
    return counter


def vertexcount(input):
    aoi_resp = input
    if vdepth(aoi_resp["geometry"]["coordinates"]) == 0:
        return str(len(aoi_resp["geometry"]["coordinates"][0]))
    elif vdepth(aoi_resp["geometry"]["coordinates"]) == 1:
        return str(len(aoi_resp["geometry"]["coordinates"][0][0]))
    else:
        print("Please check GeoJSON: Could not parse coordinates")


def order(**kwargs):
    for key, value in kwargs.items():
        if key == "name":
            dbundle["name"] = value
        if key == "sid":
            if value is not None:
                dbundle["subscription_id"] = int(value)
            else:
                dbundle.pop("subscription_id", None)
        if key == "item":
            dbundle["products"][0]["item_type"] = value
        if key == "asset":
            dbundle["products"][0]["product_bundle"] = value
        if key == "idlist":
            l = []
            if value.endswith(".csv"):
                with open(value) as f:
                    try:
                        reader = csv.reader(f)
                        for row in reader:
                            item_id = row[0]
                            if str(item_id.isalpha()) == "False":
                                l.append(item_id)
                    except Exception as e:
                        print("Issue with reading: " + str(value))
            elif value.endswith(".txt"):
                with open(value) as f:
                    for line in f:
                        item_id = line.strip()
                        l.append(item_id)
            dbundle["products"][0]["item_ids"] = l
    k = dbundle
    for key, value in kwargs.items():
        if key == "op" and value != None:
            for items in value:
                if items == "clip":
                    dbundle["tools"].append(dclip)
                elif items == "toar":
                    dbundle["tools"].append(dtoar)
                elif items == "harmonize":
                    dbundle["tools"].append(dharmonize)
                elif items == "coreg":
                    dbundle["tools"].append(dcoreg)
                elif items == "format":
                    dbundle["tools"].append(dformat)
                elif items == "zip":
                    dbundle["delivery"].update(dzip["delivery"])
                elif items == "zipall":
                    dbundle["delivery"].update(dszip["delivery"])
                elif items == "email":
                    dbundle.update(demail)
                elif items == "aws":
                    dbundle["delivery"].update(daws["delivery"])
                elif items == "azure":
                    dbundle["delivery"].update(dazure["delivery"])
                elif items == "gcs":
                    dbundle["delivery"].update(dgcs["delivery"])
                elif items == "gee":
                    dbundle["delivery"].update(dgee["delivery"])
                elif items == "composite":
                    dbundle["tools"].append(dcomposite)
                elif items == "projection":
                    dbundle["tools"].append(dreproject)
                elif items == "ndvi":
                    dndvi = {"pixel_type": "32R", "ndvi": "(b4 - b3) / (b4+b3)"}
                    dbmath["bandmath"].update(dndvi)
                elif items == "gndvi":
                    dgndvi = {"pixel_type": "32R", "gndvi": "(b4 - b2) / (b4+b2)"}
                    dbmath["bandmath"].update(dgndvi)
                elif items == "ndwi":
                    dndwi = {"pixel_type": "32R", "ndwi": "(b2 - b4) / (b4+b2)"}
                    dbmath["bandmath"].update(dndwi)
                elif items == "bndvi":
                    bndvi = {"pixel_type": "32R", "bndvi": "(b4-b1)/(b4+b1)"}
                    dbmath["bandmath"].update(bndvi)
                elif items == "tvi":
                    dtvi = {"pixel_type": "32R", "tvi": "((b4-b3)/(b4+b3)+0.5) ** 0.5"}
                    dbmath["bandmath"].update(dtvi)
                elif items == "osavi":
                    dosavi = {
                        "pixel_type": "32R",
                        "osavi": "1.16 * (b4-b3)/(b4+b3+0.16)",
                    }
                    dbmath["bandmath"].update(dosavi)
                elif items == "evi2":
                    devi2 = {
                        "pixel_type": "32R",
                        "evi2": "2.5 * (b4 - b3) / ((b4 + (2.4* b3) + 1))",
                    }
                    dbmath["bandmath"].update(devi2)
                elif items == "sr":
                    dsr = {"pixel_type": "32R", "sr": "(b4/b3)"}
                    dbmath["bandmath"].update(dsr)
                elif items == "msavi2":
                    dmsavi2 = {
                        "pixel_type": "32R",
                        "msavi2": "(2 * b4 - ((2 * b4 + 1) ** 2 - 8 * (b4 - b3)) ** 0.5) / 2",
                    }
                    dbmath["bandmath"].update(dmsavi2)
                elif items == "compression":
                    dbundle["tools"].append(dtiff)
            # dbundle[]
    for key, value in kwargs.items():
        if key == "boundary" and value != None:
            for items in k["tools"]:
                if items.get("clip"):
                    try:
                        if value.endswith(".geojson"):
                            aoi_resp = multipoly(value)
                            try:
                                for things in aoi_resp["features"]:
                                    ovall.append(things["geometry"]["coordinates"])
                            except Exception:
                                for things in json.loads(aoi_resp)["features"]:
                                    ovall.append(things["geometry"]["coordinates"])

                            aoi_geom = ovall
                            if (
                                list_depth(ovall) == 2
                                and json.loads(aoi_resp)["features"][0]["geometry"][
                                    "type"
                                ]
                                == "MultiPolygon"
                            ):
                                items["clip"]["aoi"]["coordinates"] = aoi_geom[0]
                            elif (
                                list_depth(ovall) == 2
                                and json.loads(aoi_resp)["features"][0]["geometry"][
                                    "type"
                                ]
                                != "MultiPolygon"
                            ):
                                aoi_geom = ovall[0][0]
                                items["clip"]["aoi"]["type"] = "Polygon"
                                items["clip"]["aoi"]["coordinates"] = aoi_geom
                            elif list_depth(ovall) == 1:
                                aoi_geom = ovall[0]
                                items["clip"]["aoi"]["type"] = "Polygon"
                                items["clip"]["aoi"]["coordinates"] = aoi_geom
                            elif list_depth(ovall) == 0:
                                aoi_geom = ovall
                                items["clip"]["aoi"]["type"] = "Polygon"
                                items["clip"]["aoi"]["coordinates"] = aoi_geom
                            else:
                                print(
                                    "Please check GeoJSON: Could not parse coordinates"
                                )
                            try:
                                temp = shape(items.get("clip")["aoi"])
                            except Exception as e:
                                print(e)
                            if not temp.is_valid:
                                print(
                                    "Solving shapely self intersection for multipolygon"
                                )
                                temp = temp.buffer(0)
                                geojson_string = json.dumps(
                                    shapely.geometry.mapping(temp)
                                )
                                items["clip"]["aoi"]["coordinates"] = json.loads(
                                    geojson_string
                                )["coordinates"]
                                ft = {
                                    "type": "Feature",
                                    "geometry": {"type": "Polygon", "coordinates": []},
                                    "properties": {},
                                }
                                ft["geometry"]["coordinates"] = json.loads(
                                    geojson_string
                                )["coordinates"]
                                print(
                                    "Total number of vertices in geometry: "
                                    + str(vertexcount(ft))
                                )
                                if int(vertexcount(ft)) > 500:
                                    print(
                                        "Simplifying geometry since Ordersv2 will only accept upto 500 vertex points"
                                    )
                                    b = vw.simplify_feature(ft, number=495)
                                    print(
                                        "Total number of vertices in simplified geometry: "
                                        + str(vertexcount(b))
                                        + "\n"
                                    )
                                    items["clip"]["aoi"]["coordinates"] = b["geometry"][
                                        "coordinates"
                                    ]
                                    temp = shape(b["geometry"])
                                    if not temp.is_valid:
                                        temp = temp.buffer(0)
                                        geojson_string = json.dumps(
                                            shapely.geometry.mapping(temp)
                                        )
                                        items["clip"]["aoi"][
                                            "coordinates"
                                        ] = json.loads(geojson_string)["coordinates"]
                                    elif temp.is_valid:
                                        items["clip"]["aoi"]["coordinates"] = b[
                                            "geometry"
                                        ]["coordinates"]
                        elif value.endswith(".json"):
                            with open(value) as aoi:
                                aoi_resp = json.load(aoi)
                                items["clip"]["aoi"]["coordinates"] = aoi_resp[
                                    "config"
                                ][0]["config"]["coordinates"]
                        elif value.endswith(".kml"):
                            getcoord = kml2coord(value)
                            items["clip"]["aoi"]["coordinates"] = getcoord
                    except Exception as e:
                        print(e)
                        # print('Could not parse geometry')
        #         #print(e)
    for key, value in kwargs.items():
        if key == "aws" and value != None:
            with open(value, "r") as ymlfile:
                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                for section in cfg:
                    k["delivery"]["amazon_s3"]["bucket"] = cfg["amazon_s3"]["bucket"]
                    k["delivery"]["amazon_s3"]["aws_region"] = cfg["amazon_s3"][
                        "aws_region"
                    ]
                    k["delivery"]["amazon_s3"]["aws_access_key_id"] = cfg["amazon_s3"][
                        "aws_access_key_id"
                    ]
                    k["delivery"]["amazon_s3"]["aws_secret_access_key"] = cfg[
                        "amazon_s3"
                    ]["aws_secret_access_key"]
                    k["delivery"]["amazon_s3"]["path_prefix"] = cfg["amazon_s3"][
                        "path_prefix"
                    ]
    for key, value in kwargs.items():
        if key == "azure" and value != None:
            with open(value, "r") as ymlfile:
                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                for section in cfg:
                    k["delivery"]["azure_blob_storage"]["account"] = cfg["azure"][
                        "account"
                    ]
                    k["delivery"]["azure_blob_storage"]["container"] = cfg["azure"][
                        "container"
                    ]
                    k["delivery"]["azure_blob_storage"]["sas_token"] = cfg["azure"][
                        "sas_token"
                    ]
                    k["delivery"]["azure_blob_storage"][
                        "storage_endpoint_suffix"
                    ] = cfg["azure"]["storage_endpoint_suffix"]
                    k["delivery"]["azure_blob_storage"]["path_prefix"] = cfg["azure"][
                        "path_prefix"
                    ]
    for key, value in kwargs.items():
        if key == "gcs" and value != None:
            with open(value, "r") as ymlfile:
                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                for section in cfg:
                    k["delivery"]["google_cloud_storage"]["bucket"] = cfg["gcs"][
                        "bucket"
                    ]
                    k["delivery"]["google_cloud_storage"]["credentials"] = cfg["gcs"][
                        "credentials"
                    ]
                    k["delivery"]["google_cloud_storage"]["path_prefix"] = cfg["gcs"][
                        "path_prefix"
                    ]
    for key, value in kwargs.items():
        if key == "gee" and value != None:
            project = value.split(',')[0]
            collection = value.split(',')[-1]
            k["delivery"]["google_earth_engine"]["project"] = project
            k["delivery"]["google_earth_engine"]["collection"] = collection

    for key, value in kwargs.items():
        if key == "compression" and value != None:
            for items in k["tools"]:
                if items.get("tiff_optimize"):
                    items["tiff_optimize"]["compression"] = value
    for key, value in kwargs.items():
        if key == "kernel" and value != None:
            for items in k["tools"]:
                if items.get("reproject"):
                    items["reproject"]["kernel"] = value
    for key, value in kwargs.items():
        if key == "projection" and value != None:
            for items in k["tools"]:
                if items.get("reproject"):
                    items["reproject"]["projection"] = value
    for key, value in kwargs.items():
        if key == "anchor" and value != None:
            for items in k["tools"]:
                if items.get("coregister"):
                    items["coregister"]["anchor_item"] = value
    for key, value in kwargs.items():
        if key == "format" and value != None:
            for items in k["tools"]:
                if items.get("file_format"):
                    items["file_format"]["format"] = value
    bnames = []
    for items in dbmath["bandmath"]:
        if items != "pixel_type":
            bnames.append(items)

    rg = len(bnames)
    if rg < 6:
        dck = ["b" + str(el) for el in range(1, rg + 1)]  # get serialized bands
        plist = [list(pair) for pair in zip(dck, bnames)]
        x.field_names = ["Band Number", "Band Name"]
        for items in plist:
            i = items[0]
            f = items[1]
            x.add_row([i, f])
            dbmath["bandmath"][i] = dbmath["bandmath"].pop(f)
    else:
        print("You can only use upto 5 bands")
        sys.exit()

    if len(dbmath["bandmath"]) > 0:
        k["tools"].append(dbmath)
    json_data = json.dumps(k)
    payload = json_data
    if len(bnames):
        print("\n")
        print(x)
        print("\n")

    # print('')
    # print(dbmath)

    #print(payload)
    ordname = k["name"]
    payload = payload.replace(
        "{{name}}_{{order_id}}.zip",
        "{{name}}_{{order_id}}_" + str(date.today()) + ".zip",
    )
    # print(payload)
    headers = {"content-type": "application/json", "cache-control": "no-cache"}
    response = requests.request(
        "POST", url, data=payload, headers=headers, auth=(PL_API_KEY, "")
    )
    if response.status_code == 202:
        content = response.json()
        try:
            clipboard.copy(str(url) + "/" + str(content["id"]))
            print(
                "Order created at "
                + str(url)
                + "/"
                + str(content["id"] + " and url copied to clipboard")
            )
            return str(url) + "/" + str(content["id"])
        except Exception:
            print(
                "Headless Setup: Order created at "
                + str(url)
                + "/"
                + str(content["id"])
            )
    elif response.status_code == 400:
        print("Failed with response: Bad request")
        print(response.json()["general"][0]["message"])
    elif response.status_code == 401:
        print("Failed with response: Forbidden")
    elif response.status_code == 409:
        print("Failed with response: MaxConcurrency")
    else:
        print(response.text)



from __future__ import print_function

__copyright__ = """

    Copyright 2019 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"


import requests
import json
import time
import clipboard
import sys
from datetime import date
from planet.api.auth import find_api_key

# Get Planet API and Authenticate SESSION
try:
    PL_API_KEY = find_api_key()
except:
    print("Failed to get Planet Key")
    sys.exit()
SESSION = requests.Session()
SESSION.auth = (PL_API_KEY, "")

base_payload = {
    "name": [],
    "order_type": "partial",
    "products": [],
    "tools": [],
    "delivery": {},
}

order_url = "https://api.planet.com/compute/ops/orders/v2"

idlist = []


def reorder(url, notification):
    submitted_order = SESSION.get(url)
    if submitted_order.status_code == 200 and (
        submitted_order.json()["state"] == "partial"
        or submitted_order.json()["state"] == "success"
    ):
        payload = submitted_order.json()
        for files in payload["_links"]["results"]:
            if files["name"].endswith("manifest.json"):
                time.sleep(0.2)
                resp = SESSION.get(files["location"]).json()
                for items in resp["files"]:
                    if items["path"].endswith(".tif"):
                        idlist.append(items["annotations"]["planet/item_id"])
        for items in payload["products"]:
            if not len(list(set(items["item_ids"]) - set(idlist))) == 0:
                items["item_ids"] = list(set(items["item_ids"]) - set(idlist))
                print(
                    "Submitting a total of {} items".format(
                        len(list(set(items["item_ids"]) - set(idlist)))
                    ),
                    end="\r",
                )
            else:
                print(
                    "All items were delivered successfully resubmitting complete order",
                    end="\r",
                )
                items["item_ids"] = list(set(items["item_ids"]))

        print("")
        run_complete = input("Proceed with complete reorder: y/n ")
        if run_complete == "y":
            pass
        else:
            sys.exit("Exiting application now")
        if notification is not None and notification == "email":
            base_payload.update({"notifications": {"email": True}})
        today = date.today()
        base_payload["name"] = payload["name"].replace(
            payload["name"], payload["name"] + "-resubmit-" + str(today)
        )
        base_payload["products"] = payload["products"]
        if "tools" in payload:
            base_payload["tools"] = payload["tools"]
        else:
            base_payload.pop("tools", None)
        if "delivery" in payload:
            base_payload["delivery"] = payload["delivery"]
        if "notifications" in payload:
            base_payload["notifications"] = payload["notifications"]
        payload_final = json.dumps(base_payload)
        time.sleep(1)
        headers = {"content-type": "application/json", "cache-control": "no-cache"}
        response = SESSION.post(order_url, data=payload_final, headers=headers)
        if response.status_code == 202:
            content = response.json()
            try:
                clipboard.copy(str(order_url) + "/" + str(content["id"]))
                print(
                    "Order created at "
                    + str(order_url)
                    + "/"
                    + str(content["id"] + " and url copied to clipboard")
                )
                return str(order_url) + "/" + str(content["id"])
            except Exception:
                print(
                    "Headless Setup: Order created at "
                    + str(order_url)
                    + "/"
                    + str(content["id"])
                )
        elif response.status_code == 400:
            print("Failed with response: Bad request")
            print(response.json()["general"][0]["message"])
        elif response.status_code == 401:
            print("Failed with response: Forbidden")
        elif response.status_code == 409:
            print("Failed with response: MaxConcurrency")
        else:
            print(response.text)
    else:
        print("Order has status: {}".format(submitted_order.json()["state"]))


# reorder(url='https://api.planet.com/compute/ops/orders/v2/65295800-5ca8-4560-acba-11b2e8cb7f1a',notification='email')



from __future__ import print_function

__copyright__ = """

    Copyright 2021 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import json
import csv
import os
import pyproj
from .kml2ee import kml2coord
from planet import api
from planet.api import filters
from planet.api.auth import find_api_key
from shapely.geometry import shape
from shapely.ops import transform
from datetime import datetime
from datetime import timezone
from time import mktime
from planet.api.utils import strp_lenient

try:
    PL_API_KEY = find_api_key()
except Exception as e:
    print("Failed to get Planet Key")
    sys.exit()

client = api.ClientV1(PL_API_KEY)


temp = {"coordinates": [], "type": "MultiPolygon"}
tempsingle = {"coordinates": [], "type": "Polygon"}
stbase = {"config": [], "field_name": [], "type": "StringInFilter"}
rbase = {"config": {"gte": [], "lte": []}, "field_name": [], "type": "RangeFilter"}


# Get time right
def time2epoch(st):
    str_time = datetime.strptime(st.isoformat(), "%Y-%m-%dT%H:%M:%S")
    str_tuple = str_time.timetuple()
    epoch_time = mktime(str_tuple)
    return epoch_time


def time2utc(st):
    st_time = strp_lenient(st)
    if st_time is not None:
        dt_ts = datetime.fromtimestamp(time2epoch(st_time), tz=timezone.utc)
        return dt_ts.isoformat().replace("+00:00", "Z")
    else:
        sys.exit("Could not parse time {}: check and retry".format(st))


# get coordinates list depth
def list_depth(dic, level=1):
    counter = 0
    str_dic = str(dic)
    if "[[[[[" in str_dic:
        counter += 2
    elif "[[[[" in str_dic:
        counter += 1
    elif "[[[" in str_dic:
        counter += 0
    return counter


# Area lists
ar = []
far = []
ovall = []


## Handle MultiPolygon
def multipoly(poly):
    multipoly_empty = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"id": 0},
                "geometry": {"type": "MultiPolygon", "coordinates": []},
            }
        ],
    }
    poly_list = []
    with open(poly) as jsonfile:
        data = json.load(jsonfile)
        if len(data["features"]) > 1:
            for things in data["features"]:
                poly_list.append(things["geometry"]["coordinates"])
            multipoly_empty["features"][0]["geometry"]["coordinates"] = poly_list
            return json.dumps(multipoly_empty)
        elif len(data["features"]) == 1:
            return data


# Function to use the client and then search
def idl(**kwargs):
    for key, value in kwargs.items():
        if key == "infile" and value is not None:
            infile = value
            try:
                if infile.endswith(".geojson"):
                    aoi_resp = multipoly(infile)
                    try:
                        for things in aoi_resp["features"]:
                            ovall.append(things["geometry"]["coordinates"])
                    except Exception:
                        for things in json.loads(aoi_resp)["features"]:
                            ovall.append(things["geometry"]["coordinates"])
                    # print(list_depth(ovall))
                    aoi_geom = ovall
                    if (
                        list_depth(ovall) == 2
                        and json.loads(aoi_resp)["features"][0]["geometry"]["type"]
                        == "MultiPolygon"
                    ):
                        temp["coordinates"] = aoi_geom[0]
                    elif (
                        list_depth(ovall) == 2
                        and json.loads(aoi_resp)["features"][0]["geometry"]["type"]
                        != "MultiPolygon"
                    ):
                        aoi_geom = ovall[0][0]
                        temp["type"] = "Polygon"
                        temp["coordinates"] = aoi_geom
                    elif list_depth(ovall) == 1:
                        aoi_geom = ovall[0]
                        temp["type"] = "Polygon"
                        temp["coordinates"] = aoi_geom
                    elif list_depth(ovall) == 0:
                        aoi_geom = ovall
                        temp["type"] = "Polygon"
                        temp["coordinates"] = aoi_geom
                    else:
                        print("Please check GeoJSON: Could not parse coordinates")
                elif infile.endswith(".json"):
                    with open(infile) as aoi:
                        aoi_resp = json.load(aoi)
                        aoi_geom = aoi_resp["config"][0]["config"]["coordinates"]
                elif infile.endswith(".kml"):
                    getcoord = kml2coord(infile)
                    aoi_geom = getcoord["features"][0]["geometry"]["coordinates"]
            except Exception as e:
                print("Could not parse geometry")
                print(e)
        if key == "item" and value is not None:
            try:
                item = value
            except Exception as e:
                sys.exit(e)
        if key == "start" and value is not None:
            try:
                start = time2utc(value)
                st = filters.date_range("acquired", gte=start)
            except Exception as e:
                sys.exit(e)
        if key == "end" and value is not None:
            try:
                end = time2utc(value)
                ed = filters.date_range("acquired", lte=end)
            except Exception as e:
                sys.exit(e)
        if key == "asset" and value is not None:
            try:
                asset = value
            except Exception as e:
                sys.exit(e)
        if key == "cmin":
            if value == None:
                try:
                    cmin = 0
                except Exception as e:
                    print(e)
            if value is not None:
                try:
                    cmin = float(value)
                except Exception as e:
                    print(e)
        if key == "cmax":
            if value == None:
                try:
                    cmax = 1
                except Exception as e:
                    print(e)
            elif value is not None:
                try:
                    cmax = float(value)
                except Exception as e:
                    print(e)
        if key == "num":
            if value is not None:
                num = value
            elif value == None:
                num = 1000000
        if key == "outfile" and value is not None:
            outfile = value
            try:
                open(outfile, "w")
            except Exception as e:
                sys.exit(e)
        if key == "ovp":
            if value is not None:
                ovp = int(value)
            elif value == None:
                ovp = 0.01
        if key == "filters" and value is not None:
            for items in value:
                ftype = items.split(":")[0]
                if ftype == "string":
                    try:
                        fname = items.split(":")[1]
                        fval = items.split(":")[2]
                        # stbase={'config': [], 'field_name': [], 'type': 'StringInFilter'}
                        stbase["config"] = fval.split(",")  # fval
                        stbase["field_name"] = fname
                    except Exception as e:
                        print(e)
                elif ftype == "range":
                    fname = items.split(":")[1]
                    fgt = items.split(":")[2]
                    flt = items.split(":")[3]
                    # rbase={'config': {'gte': [], 'lte': []},'field_name': [], 'type': 'RangeFilter'}
                    rbase["config"]["gte"] = int(fgt)
                    rbase["config"]["lte"] = int(flt)
                    rbase["field_name"] = fname

    print("Running search for a maximum of: " + str(num) + " assets")
    l = 0
    [head, tail] = os.path.split(outfile)
    sgeom = filters.geom_filter(temp)
    aoi_shape = shape(temp)
    if not aoi_shape.is_valid:
        aoi_shape = aoi_shape.buffer(0)
        # print('Your Input Geometry is invalid & may have issues:A valid Polygon may not possess anyoverlapping exterior or interior rings.'+'\n')
    date_filter = filters.date_range("acquired", gte=start, lte=end)
    cloud_filter = filters.range_filter("cloud_cover", gte=cmin, lte=cmax)
    asset_filter = filters.permission_filter(
        "assets." + str(asset.split(",")[0]) + ":download"
    )
    # print(rbase)
    # print(stbase)
    if len(rbase["field_name"]) != 0 and len(stbase["field_name"]) != 0:
        and_filter = filters.and_filter(
            date_filter, cloud_filter, asset_filter, sgeom, stbase, rbase
        )
    elif len(rbase["field_name"]) == 0 and len(stbase["field_name"]) != 0:
        and_filter = filters.and_filter(
            date_filter, cloud_filter, asset_filter, sgeom, stbase
        )
    elif len(rbase["field_name"]) != 0 and len(stbase["field_name"]) == 0:
        and_filter = filters.and_filter(
            date_filter, cloud_filter, asset_filter, sgeom, rbase
        )
    elif len(rbase["field_name"]) == 0 and len(stbase["field_name"]) == 0:
        and_filter = filters.and_filter(date_filter, cloud_filter, asset_filter, sgeom)
    item_types = [item]
    req = filters.build_search_request(and_filter, item_types)
    res = client.quick_search(req)
    for things in res.items_iter(
        1000000
    ):  # A large number as max number to check against
        try:
            all_assets = [
                assets.split(":")[0].replace("assets.", "")
                for assets in things["_permissions"]
            ]
            if things["properties"]["quality_category"] == "standard" and all(
                elem in all_assets for elem in asset.split(",")
            ):
                itemid = things["id"]
                footprint = things["geometry"]
                s = shape(footprint)
                if item.startswith("SkySat"):
                    epsgcode = "3857"
                else:
                    epsgcode = things["properties"]["epsg_code"]
                if aoi_shape.area > s.area:
                    intersect = (s).intersection(aoi_shape)
                elif s.area >= aoi_shape.area:
                    intersect = (aoi_shape).intersection(s)
                proj_transform = pyproj.Transformer.from_proj(
                    pyproj.Proj(4326), pyproj.Proj(epsgcode), always_xy=True
                ).transform  # always_xy determines correct coord order
                print(
                    "Processing "
                    + str(len(ar) + 1)
                    + " items with total area "
                    + str("{:,}".format(round(sum(far))))
                    + " sqkm",
                    end="\r",
                )
                if (
                    transform(proj_transform, (aoi_shape)).area
                    > transform(proj_transform, s).area
                ):
                    if (
                        transform(proj_transform, intersect).area
                        / transform(proj_transform, s).area
                        * 100
                    ) >= ovp:
                        ar.append(transform(proj_transform, intersect).area / 1000000)
                        far.append(transform(proj_transform, s).area / 1000000)
                        with open(outfile, "a") as csvfile:
                            writer = csv.writer(
                                csvfile, delimiter=",", lineterminator="\n"
                            )
                            writer.writerow([itemid])
                elif (
                    transform(proj_transform, s).area
                    >= transform(proj_transform, aoi_shape).area
                ):
                    if (
                        transform(proj_transform, intersect).area
                        / transform(proj_transform, aoi_shape).area
                        * 100
                    ) >= ovp:
                        ar.append(transform(proj_transform, intersect).area / 1000000)
                        far.append(transform(proj_transform, s).area / 1000000)
                        with open(outfile, "a") as csvfile:
                            writer = csv.writer(
                                csvfile, delimiter=",", lineterminator="\n"
                            )
                            writer.writerow([itemid])
            if int(len(ar)) == int(num):
                break
        except Exception as e:
            pass
        except (KeyboardInterrupt, SystemExit) as e:
            print("\n" + "Program escaped by User")
            sys.exit()
    num_lines = sum(1 for line in open(os.path.join(head, tail.split(".")[0] + ".csv")))
    print(
        "Total number of item ids written to "
        + str(
            os.path.join(head, tail.split(".")[0] + ".csv") + " ===> " + str(num_lines)
        )
    )
    print(
        "Total estimated cost to quota: "
        + str("{:,}".format(round(sum(far))))
        + " sqkm"
    )
    print(
        "Total estimated cost to quota if clipped: "
        + str("{:,}".format(round(sum(ar))))
        + " sqkm"
    )


# idl(infile=r"C:\Users\samapriya\Downloads\vertex.geojson",item='PSScene4Band',asset='analytic',cmin=0.0,cmax=0.9,start='2018-01-01',end='2019-12-31',ovp=8,num=40,outfile=r'C:\planet_demo\vertexidl.csv')
